#! /usr/bin/env python
#
#  ADMIT2:  AT centric version of ADMIT
#
#  simple container to put all of the ADMIT, BDP and AT in one file for testing
#
#  no 'import casa' or so allowed here. All package specific work needs to be delegated
#  If your AT_xxx needs casa routines, they need to be in a separate module
#
#
import sys, os, errno, fnmatch
import copy
import cPickle as pickle
import numpy as np
#import parfile
from util import aplot

import FlowManager as FM

_debug = False
#_debug = True

# ==============================================================================

def casa_argv(argv):
    """process the argv from a casarun script to a classic argv list
       such that the returned argv[0] is the casarun scriptname
    """
    lines = os.popen("casarun -c","r").readlines()
    n = int(lines[len(lines)-1].strip())
    return argv[n:]


# ==============================================================================

class ADMIT(object):
    projectid = 0
    def __init__(self, name='none', project=None):
        self.parfile   = "tas.def"       #  relic of ASTUTE, keep it for now
        self.name      = name
        self.project   = project
        self.fm        = FM.FlowManager() # Flow Manager
        self.keyval    = {}               # user dictionary
        self.pmode     = 0                # plot mode
        self.do_xml    = 0                # XML ?
        self.do_pickle = 0                # Pickle ?
        self.projects = [self]            # Virtual Projects?
        # maintain a projectid in case multiple admit's are present
        self.projectid = ADMIT.projectid
        ADMIT.projectid = ADMIT.projectid + 1
        self.debug     = False
    def __len__(self):
        # AT or P ?
        return len(self.projects)
    def __eq__(self, other):
        # not really used
        return isinstance(other, ADMIT) and vars(self) == vars(other)
    def __getitem__(self,index):
        """ 
        get access to an AT, and a BDP from there.
        """
        #print '__getitem__:',index
        #print 'str:',str(index),repr(index)
        # one index -> self.projects[index]
        # two index -> self.fm.tasks[index]
        # three index -> self.fm.tasks[index] -> bdp
        #return self.projects[index]
        return self.fm.tasks[index]
    def plotmode(self, plotmode, plottype='png'):
        """ plotmode determines if plots are saved, and in what format
            These are based on simple matplotlib diagrams
            Common output formats are png and pdf.
        """
        self.pmode = plotmode
        self.ptype = plottype
        # nasty cheat, need to formalize a safer method to talk to APlot
        aplot.APlot.pmode = plotmode
        print "plotmode: pmode=%d ptype=%s" % (self.pmode,self.ptype)
    def filemode(self, xml = 1, pickle =  0):
        """ filemode determines the output modes of files.
            It is possible to set all modes to 0, in which case there
            is no state output information saved, probably not a good
            idea. The default is just xml, nothing else.
        """
        self.do_xml     = xml
        self.do_pickle  = pickle
        print "filemode: xml=%d pickle=%d" % (self.do_xml,self.do_pickle)
    def addproject(self,p):
        """ Add another admit project to this project, expanding the
            virtual project
        """
        print 'addproject: ',len(self.projects)
        self.projects.append(p)
    def add(self, a, lot = None):
        """Add an AT to the stack of AT's this ADMIT contains
        Also adjust the mapping 
        Usually all but the first task will have a 'lot' (List of Tuples of source (task-id,bdp-id))
        @todo:  what if tuple is now (i,j,k) instead of (j,k) - if i>0 there needs to be an index
                into self.projects[]
        Returns the TaskID
        """
        # need to check if fm has been installed
        a.check()
        # task should inherit these from ADMIT
        a.pmode     = self.pmode
        a.do_pickle = self.do_pickle
        a.do_xml    = self.do_xml
        if len(a.bdp_in) != 0:
            print "WARNING WARNING: bdp_in not empty, somebody probably wrong here, did you use add() twice?"
        # now add the BDP_in's to the AT
        self.fm.add(a, lot)
        return a.taskid
    def run(self):
        """This will run those pieces of the pipeline flow
        that were deemed out of date. Perhaps it should also
        call runcheck() first?
        """
        self.fm.run()
    def runcheck(self):
        """ This will call the FlowManager and update all
        dependencies to ensure ALL dependent tasks are run
        """
        print "runcheck:"
    def info(self):
        print "ADMIT(%s): %s" % (self.name, self.project)
        for b in self.bdps:
            b.info()
    def show(self):
        print "=== ADMIT(%s): %s" % (self.name, self.project)
        self.fm.show()
        print "=== BDP's: " 
        # loop over all at's and show the BDP's
        for i in self.fm.tasks:
            t = self.fm.tasks[i]
            print "TASK: %s (%d)" % (t.name,len(t.bdp_out))
            for b in t.bdp_out:
                print "   bdp_out: %s" % b.filename
        print "BDP's:",self.query_bdp()
    def query_bdp(self,query=None):
        """ Query all BDP's in a PROJECT
        """
        lob = []
        if query == None:
            # loop over all at's and show the BDP's
            k = self.projectid 
            for pk in self.projects:
                k = pk.projectid
                for i in pk.fm.tasks:
                    t = pk.fm.tasks[i]
                    for j in range(len(t.bdp_out)):
                        lob.append( (k,i,j) )
        return lob
    def set(self,**kwargs):
        """set one or more global ADMIT parameters
        @todo  *args vs. **kwargs
        cf. the set() function in the AT, which is still a single 'key=val'
        """
        print "ADMIT: set" 
        for key,val in kwargs.items():
            print "SET: ",key,val
    def check(self):
        """ check all the BDP's in this admit, and see if they have name collisions
        of their BDPs, and identify orphaned branches of the tree
        A topological sort is needed as well, if they are not in the correct
        execution order.  See also the unix tsort(1) program 
        """
    def get(self,key):
        """get a global ADMIT parameter"""
        print "ADMIT: get=%s" % key
    def print_methods(self):
        """ print all the methods of this object and their doc string"""
        print '\n* Methods *'
        for names in dir(self):
            attr = getattr(self,names)
            if callable(attr):
                print names,':',attr.__doc__
    def print_attributes(self):
        """ print all the attributes of this object and their value """
        print '* Attributes *'
        for names in dir(self):
            attr = getattr(self,names)
            if not callable(attr):
                print names,':',attr
    def print_all(self):
        """ calls all the methods of this object """
        for names in dir(self):
            attr = getattr(self,names)
            if callable(attr) and names != 'print_all' and names != '__init__':
                attr() # calling the method
    def pdump(self,name=None):
        """ dump an admit object into pickle format
        """
        if not name:
            pname = self.name + '.p'
        else:
            pname = name
        print "ADMIT: pickle saving %s" % pname
        pickle.dump(self,open(pname,"wb"))
    def pload(self,pname):
        """ load an admit object into pickle format
            since admit stores many objects in deeply nested structures
            (the AT's and BDP's) they are duplicated. After a pickle.load()
            these should be re-pointed?
        """
        print "ADMIT: pickle loading %s" % pname
        return pickle.load(open(pname,"rb"))
    def discover(self,mode=None,rootdir='.'):
        print "query_dir() and find_files() are the worker functions"
        print "discover not implemented yet"
        pp = []
        return pp
    def query_dir(self,here=None):
        """
        from here, drill down and find directories in which ADMIT exists
        """
        dlist = []
        if here == None:
            path = "."
        else:
            path = here
        n = 0
        for path, dirs, files in os.walk(path):
            # better not to loop, but os.path() for existence
            n = n + 1
            for f in files:
                if f == self.parfile: dlist.append(path)
        if self.debug: print "Queried ",n," directories, found ",len(dlist), " with a parfile"
        return dlist
    def find_files(self, pattern="*.fits"):
        """
        Find files containing a wildcard pattern
        """
        flist = []
        for file in os.listdir('.'):
            if fnmatch.fnmatch(file,pattern):
                flist.append(file)
        return flist
    def setdir(self,dirname, create=True):
        """
        change directory to dirname to work in. Assumed to contain parameter file
        if the directory doesn't exist yet, create it
        See pushd()/popd() for a better version - deprecate this.
        """
        def mkdir_p(path):
            #if not os.path.isdir(dirname):
            #    os.makedirs(dirname)
            #
            try:
                os.makedirs(path)
            except OSError as exc: # Python >2.5
                if exc.errno == errno.EEXIST and os.path.isdir(path):
                    pass
                else: raise
        self.p = dirname
        self.pwd = os.getcwd()
        if create: mkdir_p(dirname)
        os.chdir(dirname)
        if self.debug: print "ADMIT::setdir %s" % dirname
    def tesdir(self):
        """
        revert back from previous setdir (sorry, not recursive yet)
        See pushd()/popd() for a better version
        """
        os.chdir(self.pwd)
    def walkdir(self,dlist):
        print "Walkdir ",dlist
        print "Deprecated"
        if False:
            for d in dlist:
                self.setdir(d)
                print "d: ",d
                par = pp.ParFile()
                print par.get('fits')
                print par.keys()
                self.tesdir()


if __name__ == "__main__":

    # pickle files store the module name, so keep that here, 
    # since that is how we run it externally
    import admit

    #  silly stuff to limit the flow of the pipeline (Q=0 only produces b0, Q=1 the b1, etc.)
    Q = 0
    if len(sys.argv)>1:
        Q = int(sys.argv[1])
    #
    print "TESTING admit1.py:"
    #
    a = admit.ADMIT()
    if Q>=0:
        b0 = admit.BDP_file("b0")
        a0 = admit.AT_file([],[b0])
        a0.run()
        a.add(b0)
        if Q>0:
            b1 = admit.BDP_buckett("b1")
            a1 = admit.AT_flow([b0],[b1])
            a1.run()
            a.add(b1)
            if Q>1:
                b21 = admit.BDP_buckett("b21")
                b22 = admit.BDP_buckett("b22")
                a2 = admit.AT_flow12([b1],[b21,b22])
                a2.run()
                a.add(b21)
                a.add(b22)
                if Q>2:
                    b3 = admit.BDP_buckett("b3")
                    a3 = admit.AT_flow21([b21,b22],[b3])
                    a3.run()
                    a.add(b3)
                    if Q>3:
                        n = 10
                        b4 = range(n)
                        for i in range(n):
                            id = "b4_%d" % i
                            b4[i] = admit.BDP_buckett(id)
                        a4 = admit.AT_flowN([b3],b4)
                        a4.run()
                        for i in range(n):
                            a.add(b4[i])
    # a0.pdump('test00.p')
    a.pdump('admit1.p')
    b0.pdump('b00.p')
    print "Q=%d" % Q
