"""
This is the base class for all AT types.
"""

import xml.etree.cElementTree as et
import admit.util.bdp_types as ut
from tasks import *
import types

class AT(object) :
    def __init__(self,type=None, id="", parameters={},stale=True) :
        self.type = type
        self.products = []
        self.id = id
        self.__version = "0.0.0"
        self.stale = stale
        self.parameters = parameters
        self.needToSave = False
        self.super = None

    def markUpToDate(self):
        self.stale = False

    def setSuper(self,super):
        self.super = super

    def setParameter(self,name = "",parameter={}):
        if(type(parameter) == types.DictType) :
            self.parameters = parameter
        else:
            self.parameters[name] = parameter
        self.markChanged()
    
    def markChanged(self):
        self.stale = True
    
    def addProduct(self,item):
        #print "SETTING UID",item.UID
        item.set("UID",self.id)
        #print item.UID
        self.products.append(item)
    
    def getVersion(self):
        return self.__version
    
    def write(self,root):
        node = et.SubElement(root,"AT")
        node.set("type",self.type)
        node.set("id", self.id)
        node.set("version", self.__version)
        node.set("parameters", str(self.parameters))
        if(self.stale):
            node.set("stale","1")
        else:
            node.set("stale","0")
        return root

    def trigger(self):
        if(self.super == None):
            return
        self.super.write()

    def save(self):
        if(not self.needToSave) :
            return
        for i in self.products:
            i.write()
        self.needToSave = False

    def run(self,task,args,force=False):
        if(type(args) != types.DictType) :
            raise Exception,"Arguments must be in a dictionary"
        if(not self.stale and not force) :
            print "Parameters have not changed, not running task %s, use force=True to override this" % (task)
            return
        possibles = globals().copy()
        possibles.update(locals())
        #for key in possibles:
        #    print key
        #print task
        val = possibles.get(task)(**args)
        self.needToSave = True
        #self.trigger()
        self.stale = False
        return val