from admit.at.AT import AT
import admit.util.bdp_types as ut
from admit.bdp.CubeStats_BDP import CubeStats_BDP
from admit.util.Table import Table
from copy import deepcopy
import numpy as np

class CubeStats_AT(AT):
    def __init__(self,id="", parameters={},upToDate=True):
        AT.__init__(self,ut.CUBESTATS_AT,"1234", parameters,upToDate)
        self.__version = "0.0.0"
        self.kw = {"imagename": True,
                   "axes": False,
                   "region": False,
                   "box": False,
                   "chans": False,
                   "stokes": False,
                   "listit": False,
                   "verbose": False,
                   "mask": False,
                   "stretch": False,
                   "logfile": False,
                   "append": False,
                   }
        self.parameters["listit"] = True
        self.parameters["verbose"] = True
        self.parameters["append"] = False
        self.parameters["doCube"] = True
        self.parameters["axes"] = [0,1]

    def getFreq(self,value):
        end = 0
        for i in range(-1,-5,-1):
            if(not value[i].isalpha()):
                end = i + 1
                break
        freq = float(value[:end])
        units = value[end:]
        if(units == "Hz"):
            print "Hz"
            return freq/(1.0*10**9)
        if(units == "kHz"):
            print "kHz"
            return freq/(1.0*10**6)
        if(units == "MHz"):
            print "MHz"
            return freq/(1.0*10**3)
        if(units == "GHz"):
            print "GHz"
            return freq
        if(units == "THz"):
            print "THz"
            return freq*1000.0
        raise Exception,"Unknown units detected: %s" % (units)


    def run(self):
        print "Running"
        args = {}
        for key in self.kw :
            if(key in self.parameters):
                args[key] = self.parameters[key]
            elif(self.kw[key]):
                raise Exception,"Required key word %s is missing from argument list" % (key)
        
        stats = super(CubeStats_AT, self).run("imstat",args)

        for k in stats:
            print k

        cubestats = deepcopy(CubeStats_BDP(self.id,self.parameters["imagename"] + ".cubestats" + ".xml"))
        labels = ["channel","frequency","mean","sigma","peakval"]
        units = ["number","GHz","Jy/beam","Jy/beam","Jy/beam"]

        freqs = []
        length = len(stats["max"].tolist())
        sfreq = self.getFreq(stats["blcf"].split(",")[-1])
        efreq = self.getFreq(stats["trcf"].split(",")[-1])
        step = (sfreq - efreq)/(length-1)
        for i in range(0,length):
            freqs.append(sfreq + float(i)*step)
        tmin = [length,min(freqs),np.min(stats["mean"]),np.min(stats["sigma"]),np.min(stats["max"])]
        tmax = [length,max(freqs),np.max(stats["mean"]),np.max(stats["sigma"]),np.max(stats["max"])]
        data = np.array([range(0,length),freqs,stats["mean"].tolist(),stats["sigma"].tolist(),stats["max"].tolist()])
        table = Table(labels,units,["all"],data,tmin,tmax)
        cubestats.set("stats",table)
       
        # need to select all?
        if(self.parameters["doCube"]):
            args["axes"] = -1
            stats = super(CubeStats_AT, self).run("imstat",args,force=True)
            print stats
            cubestats.set("mean",stats["mean"].tolist()[0])
            cubestats.set("sigma",stats["sigma"].tolist()[0])
            cubestats.set("maxval",stats["max"].tolist()[0])
            cubestats.set("maxpos",stats["maxpos"].tolist())

        self.addProduct(cubestats)
            
