from admit.at.AT import AT
import admit.util.bdp_types as ut
from admit.bdp.Moment_BDP import Moment_BDP
from admit.bdp.CubeSum_BDP import CubeSum_BDP
from admit.util.Image import Image
import types
from copy import deepcopy
import os
import admit.util.util as util

class Moment_AT(AT):
    def __init__(self,id="", parameters={},stale=True):
        AT.__init__(self,ut.MOMENT_AT,"1234", parameters,stale)
        self.kw = {"imagename":True,
                   "moments":False,
                   "axis":False,
                   "region":False,
                   "box":False,
                   "chans":False,
                   "stokes":False,
                   "mask":False,
                   "includepix":False,
                   "excludepix":False,
                   "outfile":True,
                   "stretch":False,
                   }
        self.__version = "0.0.0"
        if(not "moments" in self.parameters):
            self.parameters["moments"] = [0]
        self.momentFileExtensions = {-1: ".average",
                                     0:  ".integrated",
                                     1:  ".weighted_coord",
                                     2:  ".weighted_dispersion_coord",
                                     3:  ".median",
                                     4:  "",
                                     5:  ".standard_deviation",
                                     6:  ".rms",
                                     7:  ".abs_mean_dev",
                                     8:  ".maximum",
                                     9:  ".maximum_coord",
                                     10: ".minimum",
                                     11: ".minimum_coord",
                                     }
            
    def setMoments(self,moments):
        if(type(moments) == types.ListType):
            self.parameters["moments"] = moments
        else :
            self.parameters["moments"] = [moments]
        self.markChanged()

    def selectChannels(self,start,end):
        self.parameters["chans"] = "%s~%s" % (str(start),str(end))
        self.markChanged()

    def setChanRange(self,range):
        self.parameters["chans"] = range
        self.markChanged()

    def setRange(self,min=-100000.0,max=100000.0):
        self.parameters["includepix"] = [min,max]
        self.markChanged()
    
    def setCutoff(self,cutoff):
        self.setRange(min=cutoff)
        self.markChanged()

    def run(self):
        print "Running"
        args = {}
        if(len(self.parameters["moments"]) == 1):
            self.parameters["outfile"] += self.momentFileExtensions[self.parameters["moments"][0]]
            os.system("rm -rf %s" % (self.parameters["outfile"]))
        else:
            for mom in self.parameters["moments"] :
                os.system("rm -rf %s.%s" % (self.parameters["outfile"],self.momentFileExtensions[mom]))
        for key in self.kw :
            if(key in self.parameters):
                args[key] = self.parameters[key]
            elif(self.kw[key]):
                raise Exception,"Required key word %s is missing from argument list" % (key)
        
        super(Moment_AT, self).run("immoments",args)

        for mom in self.parameters["moments"]:
            moment = None
            util.casa2fits({"imagename":self.parameters["outfile"] + self.momentFileExtensions[mom],"fitsimage":self.parameters["outfile"] + self.momentFileExtensions[mom] + ".fits"})
            if(self.type == ut.MOMENT_AT):
                moment = deepcopy(Moment_BDP(self.id,self.parameters["imagename"] + ".moment_" + str(mom) + ".xml"))
            else:
                moment = deepcopy(CubeSum_BDP(self.id,self.parameters["imagename"] + ".cubesum_" + str(mom) + ".xml"))
            img = deepcopy(Image(self.parameters["outfile"] + self.momentFileExtensions[mom] + ".fits","FITS"))
            moment.set("image",img)
            moment.set("moment",mom)
            self.addProduct(moment)
            util.rmdir(self.parameters["outfile"] + self.momentFileExtensions[mom])
        