"""
This is the base class for all BDP types.
"""

import xml.etree.cElementTree as et
import types
import numpy as np
import admit.util.bdp_types as ut
from admit.util.Table import Table
from admit.util.Image import Image
from admit.xmlio.DtdReader import DtdReader
import time
import zipfile

_debug = False
_debug = True
ndarray = np.ndarray(1)

class BDP(object) :
    def __init__(self,taskid=0,xmlFile=None) :
        # just some default values
        self.project = None
        self.sous = None
        self.mous = None
        self.gous = None
        self.date = None                        # date of last edit
        self.UID = None                         # unique id
        self.type = ut.GENERIC                     # BDP type
        self.xmlFile = xmlFile                  # xml file associated with BDP
        self.updated  = False                   # has been updated
        self.taskid = taskid
        self.ctime = 0.0
        self.etime = 0.0
        self.stime = 0.0
        self.ttime = 0.0
        self.sum = 0.0
        
    def show(self):
        return self.name
    
    def update(self,new_state):
        if _debug: print "UPDATE: %s" % self.name
        self.updated = new_state
        for d in self.derv:
            d.update(new_state)
            
    def depends_on(self,other):
        if other == None: return
        if _debug: print 'BDP %s depends on %s' % (self.name,other.show())
        self.deps.append(other)
        other.derv.append(self)

    def report(self):
        print "===report==="
        print "BDP::%s" % self.name
        for d in self.deps:
            print "BDP::deps %s" % d.show()
        for d in self.derv:
            print "BDP::derv %s" % d.show()
        for t in self.task:
            print "BDP::task %s" % t.show()
 
    def set(self,key,val):
        if _debug: print "BDP::set %s to %s" % (key,val)
        setattr(self,key,val)
        
    def get(self,key):
        if _debug: print "BDP::get %s" % (key)
        try:
            return getattr(self,key)
        except :
            return None

    def addEtime(self,time):
        self.etime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
    
    def addTtime(self,time):
        self.stime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
   
    def addStime(self,time):
        self.ttime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
        
    def pTime(self,lab,time):
        #print lab,(time-self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time

    def run(self):
        if self.updated: 
            if _debug: print "BDP::%s (%s) was up to date" % (self.name,self.filename)
            return False
        if _debug: print "BDP::%s (%s) running" % (self.name,self.filename)
        if len(self.task) == 0:
            if _debug: print " Warning: no task set ???"
            # raise ?
        else:
            for t in self.task:
                t.run()
        self.updated = True
        return True
        
    def write(self,xmlFile = None) :
        """
        Method to write the BDP to an xml file
        Do not edit unless you know what you are doing as all BDP's rely on this method to work properly
        """
        global ndarray
        start = time.time()
        self.ctime = start
        dtdRead = DtdReader(self.type + ".dtd")
        order = dtdRead.getOrder()
        dtd = dtdRead.getDtd()
        self.pTime("DTD Read",time.time())
        if(xmlFile == None and self.xmlFile == None) :
            raise Exception("No output files was specified")
        if(xmlFile != None) :
            self.xmlFile = xmlFile
        root = et.Element("BDP")
        root.set("type",self.type)
        """now loop through all data members in the dtd"""
        field = {}
        self.pTime("init",time.time())
        for item in order:
            attr = getattr(self,item)
            #print "\n\n",item,attr
            #print "IN"
            #print type(item),type(attr)
            if(isinstance(item,BDP) or isinstance(attr,BDP)) :
                #print "BDP"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",attr.type)
                self.addStime(time.time())
                field[item].text = attr.xmlFile
                self.addTtime(time.time())
                attr.write()
            elif(isinstance(item,Table) or isinstance(attr,Table)) :
                #print "BDP TABLE"
                field[item] = attr.write(root,item)
            elif(isinstance(item,Image) or isinstance(attr,Image)) :
                #print "BDP IMAGE"
                field[item] = attr.write(root,item)
            elif(type(attr) == types.BooleanType) :
                #print "BOOL"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.BOOL)
                self.addStime(time.time())
                if(attr) :
                    field[item].text = "1"
                else :
                    field[item].text = "0"
                self.addTtime(time.time())
            elif(type(attr) == types.ClassType) :
                pass
            elif(type(attr) == types.FloatType) :
                #print "FLOAT"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.FLOAT)
                self.addStime(time.time())
                field[item].text = repr(attr)
                self.addTtime(time.time())
            elif(type(attr) == types.DictionaryType) :
                #print "DICT"
                ndfound = False
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                for k,v in attr :
                    if(type(v) == type(ndarray)) :
                        ndarray = True
                        attr[k] = np.ndarray.tolist(v)
                field[item].set("type",ut.DICT)
                self.addStime(time.time())
                if(ndfound) :
                    field[item].set("ndarray","1")
                else :
                    field[item].set("ndarray","0")
                field[item].text = str(attr)
                self.addTtime(time.time())
            elif(type(attr) == types.IntType) :
                #print "INT"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.INT)
                self.addStime(time.time())
                field[item].text = str(attr)
                self.addTtime(time.time())
            elif(type(attr) == types.ListType) :
                #print "LIST"
                ndfound = False
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                for i in range(0,len(attr)) :
                    if(type(attr[i]) == type(ndarray)) :
                        ndarray = True
                        attr[i] = np.ndarray.tolist(attr[i])
                field[item].set("type",ut.LIST)
                self.addStime(time.time())
                if(ndfound) :
                    field[item].set("ndarray","1")
                else :
                    field[item].set("ndarray","0")
                field[item].text = str(attr)
                self.addTtime(time.time())
            elif(type(attr) == types.LongType) :
                #print "LONG"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.LONG)
                self.addStime(time.time())
                field[item].text = str(attr)
                self.addTtime(time.time())
            elif(type(attr) == types.StringType or type(attr) == types.NoneType) :
                #print "STRING"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.STRING)
                self.addStime(time.time())
                if(type(attr) == types.NoneType) :
                    field[item].text = "None"
                else :
                    field[item].text = attr
                self.addTtime(time.time())
            elif(type(attr) == types.TupleType) :
                #print "TUPLE"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                field[item].set("type",ut.TUPLE)
                self.addStime(time.time())
                field[item].text = str(attr)
                self.addTtime(time.time())
            elif(type(attr) == type(ndarray)) :
                #print "NDARR"
                field[item] = et.SubElement(root,item)
                self.addEtime(time.time())
                attr = np.ndarray.tolist(attr)
                field[item].set("type",ut.NDARRAY)
                self.addStime(time.time())
                field[item].text = str(attr)
                self.addTtime(time.time())
            else :
                raise Exception("Unknown type %s encountered for %s" % (type(item),item))
        #print "DONE\n"
        """Return a pretty-printed XML string for the Element.
        """
        #print "\n\n WRITING\n\n"
        rough_string = et.tostring(root, 'utf-8')
        self.pTime("Rough",time.time())
        #reparsed = md.parseString(rough_string)
        #self.pTime("reparsed",time.time())
        #temp = reparsed.toprettyxml(indent="  ")
        temp = rough_string.replace(">",">\n")
        temp = temp.replace("</","\n</")
        self.pTime("Pretty",time.time())
        #head = temp.split("\n")[0]
        #self.pTime("Split",time.time())
        #temp = temp.replace(head + "\n","")
        #self.pTime("Replace",time.time())
        outFile = open(self.xmlFile,'w')
        outFile.write("<?xml version=\"1.0\" ?>\n")
        self.pTime("Wr head",time.time())
        # write out the dtd info at the top
        outFile.write("<!DOCTYPE BDP [\n\n")
        for line in dtd :
            outFile.write(line)
        outFile.write("]>\n\n")
        outFile.write(temp)
        self.pTime("Wr xml",time.time())
        outFile.close()
        #print "Etime",self.etime
        #print "Stime",self.stime
        #print "Ttime",self.ttime
        #print "Sum",self.sum
        #print "Total",time.time()-start
        #zFile = zipfile.ZipFile(self.xmlFile + ".zip",'w',compression=zipfile.ZIP_DEFLATED)
        #zFile.write(self.xmlFile)
        #self.pTime("Zip xml",time.time())
        #zFile.close()

def DependsOn(a,b):
    """ b depends on a """
    b.depends_on(a)


