"""
Class for IMAGE entries in BDP's
"""

import xml.etree.cElementTree as et
import admit.util.bdp_types as ut

class Image(object):
    def __init__(self,fileName = None,fileType = None,xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,desc=""):
        self.fileName = fileName        # image file name
        self.fileType = fileType        # image file type e.g. PNG, FITS, ...
        self.xmin = xmin                # lowest x value
        self.xmax = xmax                # highest x value
        self.ymin = ymin                # lowest y value
        self.ymax = ymax                # highest y value
        self.description = desc         # image description/caption

    def write(self,root,name):
        """
        Method to write an xml entry for an Image
        """
        node = et.SubElement(root,name)
        node.set("type",ut.IMAGE)
        fln = et.SubElement(node,"fileName")
        fln.set("type",ut.STRING)
        fln.text = self.fileName
        xmin = et.SubElement(node,"xmin")
        xmin.set("type",ut.FLOAT)
        xmin.text = str(self.xmin)
        xmax = et.SubElement(node,"xmax")
        xmax.set("type",ut.FLOAT)
        xmax.text = str(self.xmax)
        ymin = et.SubElement(node,"ymin")
        ymin.set("type",ut.FLOAT)
        ymin.text = str(self.ymin)
        ymax = et.SubElement(node,"ymax")
        ymax.set("type",ut.FLOAT)
        ymax.text = str(self.ymax)
        desc = et.SubElement(node,"description")
        desc.set("type",ut.STRING)
        desc.text = str(self.ymax)
        
        return node
        
    # the following are just basic set/get methods for the individual data members
    def set(self,key,val):
        setattr(self,key,val)
        
    def get(self,key):
        try:
            return getattr(self,key)
        except :
            return None

    def setFileName(self,name):
        self.fileName = name
    
    def getFileName(self):
        return self.fileName
    
    def setLimits(self,xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0):
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax
    
    
