"""
Class for IMAGE entries in BDP's
"""

import numpy as np
import admit.util.bdp_types as ut
import types
import xml.etree.cElementTree as et


class Table(object):
    def __init__(self, columns = [], units = [], planes = [], data = None, tmin = [], tmax = []):
        self.columns = columns  # column labels
        self.units = units      # units of columns
        self.planes = planes    # label for planes
        if(data == None) :
            self.data = np.array([])
        else :
            self.data = data    # actual data
        self.min = tmin         # minimum value for each column
        self.max = tmax         # maximum value for each column
    
    def exportTable(self,fileName,plane = 0):
        """
        Method to export a table to ascii text
        """
        if(len(self.data.shape) == 2) :
            f = open(fileName,'w')
            line = "#|"
            for i in self.columns :
                line += i + "\t|"
            f.write(line + "\n")
            x,y = self.data.shape
            for i in range(0,x) :
                line = ""
                for j in range(0,y) :
                    line += str(self.data[i][j]) + "\t"
                f.write(line + "\n")
            f.close()

        elif(len(self.data.shape) == 3) :
            f = open(fileName,'w')
            line = "#|"
            for i in self.columns :
                line += i + "\t|"
            f.write(line + "\n")
            x,y,z = self.data.shape
            if(plane > z) :
                raise Exception, "Requested plane does not exist in table"
            for i in range(0,x) :
                line = ""
                for j in range(0,y) :
                    line += str(self.data[i][j][plane]) + "\t"
                f.write(line + "\n")
            f.close()
        else :
            print "Cannot write out this table at this time."
    
    def write(self,root,name):
        """
        Method to write an xml entry for a Admit_table
        """
        node = et.SubElement(root,name)
        node.set("type",ut.TABLE)
        columns = et.SubElement(node,"columns")
        columns.set("type",ut.LIST)
        columns.set("ndarray","0")
        columns.text = str(self.columns)
        units = et.SubElement(node,"units")
        units.set("type",ut.LIST)
        units.set("ndarray","0")
        units.text = str(self.planes)
        planes = et.SubElement(node,"planes")
        planes.set("type",ut.LIST)
        planes.set("ndarray","0")
        planes.text = str(self.planes)
        data = et.SubElement(node,"data")
        data.set("type",ut.NDARRAY)
        data.text = str(np.ndarray.tolist(self.data))
        dmin = et.SubElement(node,"min")
        dmin.set("type",ut.LIST)
        dmin.set("ndarray","0")
        dmin.text = str(self.min)
        dmax = et.SubElement(node,"max")
        dmax.set("type",ut.LIST)
        dmax.set("ndarray","0")
        dmax.text = str(self.max)

        return node

    def plotter(self,x,y,title=None,figname=None,xlab=None,ylab=None):
        """simple plotter of multiple columns against one column"""
        #plt.ioff()
        #self.fign = self.fign + 1
        #fig = plt.figure(self.fign)
        #ax1 = fig.add_subplot(1,1,1)
        #for yi in y:
        #    ax1.plot(x,yi)
        #if title:    ax1.set_title(title)
        #if xlab:     ax1.set_xlabel(xlab)
        #if ylab:     ax1.set_ylabel(ylab)
        #if figname: 
        #    fig.savefig(figname)
        #plt.show()
        pass
        
    def histogram(self,x,title=None,figname=None,xlab=None,hrange=None,bins=80):
        """simple histogram of one or more columns """
        #self.fign = self.fign + 1
        #fig = plt.figure(self.fign)
        #ax1 = fig.add_subplot(1,1,1)
        #for xi in x:
        #    if range:
        #        ax1.hist(xi,bins=bins,range=range)
        #    else:
        #        ax1.hist(xi,bins=bins)
        #if title:    ax1.set_title(title)
        #if xlab:     ax1.set_xlabel(xlab)
        #if figname: 
        #    fig.savefig(figname)
        #plt.show()
        pass

    # the following are just basic set/get methods for the individual data members
    def setUID(self,uid):
        self.UID = uid
    
    def getUID(self):
        return self.UID
    
    def setColumns(self,cols):
        self.columns = cols
    
    def getColumns(self):
        return self.columns
    
    def getPlanes(self):
        return self.planes
    
    # these methods allow for the addition of columns, rows, and planes to existing tables
    def addColumn(self,col,data):
        self.columns.append(col)
        if(type(data) == types.ListType) :
            data = np.array([data])
        self.data.concatenate((self.data,data.T),axis=1)
    
    def addRow(self,row):
        if(type(row) == types.ListType) :
            row = np.array([row])
        self.data.concatenate((self.data,row),axis=0)
        
    def addPlane(self,plane,data):
        self.planes.append(plane)
        if(type(data) == types.ListType) :
            data = np.array([data])
        self.data.concatenate((self.data,data),axis=2)

    def setData(self,data):
        if(type(data) == types.ListType) :
            data = np.array(data)
        self.data = data
    
    def getColumnByName(self,name):
        try :
            i = self.columns.index(name)
            return self.data.T[i]
        except :
            pass
    
    def getColumn(self,col):
        return self.data.T[col]
    
    def getPlane(self,pln):
        pass
        #return self.data[][][pln]
    
    def setMin(self,vals):
        # detect type
        pass
    
    def setMax(self,vals):
        #detect type
        pass
    
    def getMin(self,col=-1):
        if(col > -1) :
            return self.min[col]
        return self.min
    
    def getMax(self,col=-1):
        if(col > -1) :
            return self.max[col]
        return self.max
