"""
  This module is for parsing the input xml files, both admit.xml and the general band xml files
"""
from xml import sax
import ast
import copy
import numpy as np
import types
from admit.bdp import *
from admit.util.bdp_types import *
import admit.util.registry as registry
from admit.util import Table,Image
import time
import os

ndarray = np.ndarray(1)

class Parser(object):
    """
    Main parser class. Use as follows:
    p = Parser.Parser(xmlFile)
    b = p.parse()
    
    b will contain a populated instance of the BDP sub-class contained in the given xml file
    """
    def __init__(self,baseDir = "",xmlFile = ""):
        self.xmlFile = xmlFile          # the xml file to parse
        self.ATparser = sax.make_parser() # the xml sax parser
        self.baseDir = baseDir
        self.tasks = None
        self.userData = None

    def getTasks(self):
        return self.tasks
    
    def getUserData(self):
        return self.userData

    def getFiles(self):
        files = []
        for file in os.listdir(self.baseDir):
            if(file.endswith(".xml") and not "admit.xml" in file):
                files.append(file)
        return files

    def addBDPtoAT(self,bdp):
        for at in self.tasks :
            if(at.id == bdp.UID):
                at.addProduct(copy.deepcopy(bdp))

    def parse(self,doParse = True):     # rework to take admit.xml and parse all xml files
        #print "PARSING",doParse
        if(self.xmlFile != None) :
            print "GO"
            self.ATContentHandler = ATParser(self.baseDir + self.xmlFile)
            self.ATErrorHandler = AT_ErrorHandler()
            self.ATparser.setContentHandler(self.ATContentHandler)
            if(doParse) :
                self.ATparser.parse(open(self.baseDir + self.xmlFile))
                self.tasks = self.ATContentHandler.getAT()
                self.userData = self.ATContentHandler.getUserData()
                files = self.getFiles()
                for file in files:
                    BDPparser = sax.make_parser()
                    BDPContentHandler = BDPParser(self.baseDir + file)
                    BDPErrorHandler = BDP_ErrorHandler()
                    BDPparser.setContentHandler(BDPContentHandler)
                    BDPparser.setErrorHandler(BDPErrorHandler)
                    # do the actual parsing
                    BDPparser.parse(open(self.baseDir + file))
                    # return the generated BDP class
                    self.addBDPtoAT(BDPContentHandler.getBDP())

class BDPParser(sax.handler.ContentHandler) :
    """
        This is the customized parser for all BDP sub-classes
    """
    def __init__(self,xmlFile) :
        self.start = time.time()
        self.ctime = time.time()
        self.etime = 0.0
        self.stime = 0.0
        self.ttime = 0.0
        self.sum = 0.0
        sax.handler.ContentHandler.__init__(self)
        self.pTime("Init",time.time())
        self.dtd = DTDParser(xmlFile)       # get the dtd info
        self.dtd.parse()
        self.pTime("DTD",time.time())
        self.BDP = None                     # an instance of the BDP sub-class
        self.type = None                    # the type of the BDP sub-class
        self.name = None                    # name of the current node
        self.ndarr = False                  # if the current node is an ndarray
        self.Image = None                   # instance of Image class for parser
        self.Table = None                   # instance of Admit_table class for parser
        self.Task = None                    # instance of Task class for parser
        self.inImage = False                # are we currently parsing an Image node
        self.inTable = False                # are we currently parsing a Admit_table node
        self.inTask = False                 # are we currently parsing a Task node
        self.imageName = ""
        self.taskName = ""
        self.tableName = ""
        
    def addEtime(self,time):
        self.etime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
    
    def addTtime(self,time):
        self.ttime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
   
    def addStime(self,time):
        self.stime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
        
    def pTime(self,lab,time):
        print lab,(time-self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time

    def getBDP(self):
        # return the generated BDP
        return copy.deepcopy(self.BDP)

    def startElement(self, name, attrib) :
        """
        Method called whenever a new element is found
        dtd validation is done for each node and attribute
        input: name - the name of the element
               attrib - list of attributes found
        """
        # get the type of the data
        temp = attrib.get("type")
        self.dtd.check(name, "type", temp)
        self.name = name
        #print "\nNODE",name,temp
        if(temp == STRING) :
            self.type = types.StringType
        elif(temp == BOOL) :
            self.type = types.BooleanType
        elif(temp == FLOAT) :
            self.type = types.FloatType
        elif(temp == LONG) :
            self.type = types.LongType
        elif(temp == DICT) :
            self.type = types.DictType
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            try :
                if(attrib.get(int("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == LIST) :
            self.type = types.ListType
            #print "LIST",name
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            #print "LIST",name
            try :
                if(int(attrib.get("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == INT) :
            self.type = types.IntType
        elif(temp == TUPLE) :
            self.type = types.TupleType
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            try :
                if(int(attrib.get("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == NDARRAY) :
            self.type = type(ndarray)
        elif(temp == IMAGE) :
            self.Image = copy.deepcopy(Image.Image())
            self.inImage = True
            self.imageName = name
        elif(temp == TABLE) :
            self.Table = copy.deepcopy(Table.Table())
            self.inTable = True
            self.tableName = name
        elif(name == BDP and temp in registry.BDPTYPES) :
            self.type = temp
            self.BDP = copy.deepcopy(registry.BDPTYPES[temp])
            #print "done start",temp,self.BDP
                
    def endElement(self, name) :
        """
        Method called whenever the end of an xml element is reached
        input: name - the name of the element
        """
        # just reset the tracking stuff
        #print "end element",name
        self.type = None
        self.name = None
        self.ndarr = False
        if(name == self.tableName) :
            setattr(self.BDP,self.tableName,copy.deepcopy(self.Table))
            self.inTable = False
            self.tableName = ""
        elif(name == self.imageName) :
            setattr(self.BDP,self.imageName,copy.deepcopy(self.Image))
            self.inImage = False
            self.imageName = ""
        elif(name == self.taskName) :
            setattr(self.BDP,self.taskName,copy.deepcopy(self.Task))
            self.inTask = False
            self.taskName = ""
        elif(name == BDP) :
            # one last validation run
            self.pTime("Start valid",time.time())
            if(not self.dtd.checkAll()) :
                print "Some required nodes missing from xml file, attempting to continue anyway"
            self.pTime("End valid",time.time())
            print "Etime",self.etime
            print "Stime",self.stime
            print "Ttime",self.ttime
            print "Sum",self.sum
            print "Total",time.time()-self.start

    
    def characters(self, ch) :
        """
        Method called whenever characters are detected in an xml node
        This method does some dtd validation
        input: ch - the characters detected
        """
        target = None
        #print type(ch)
        char = str(ch).strip()
        #print "characters",char,"XXX"
        if(char.isspace() or not char) :
            #print "RETURNING"
            return
        # handle image, table, and task nodes specially
        if(self.inImage):
            target = self.Image
        elif(self.inTable):
            target = self.Table
        elif(self.inTask):
            target = self.Task
        else :
            target = self.BDP
        # a list or dictionary has to be decoded
        if(self.type == types.ListType or self.type == types.DictType or self.type == types.TupleType) :
            temp = ast.literal_eval(char)
            # if list/dict contains an numpy.ndarray it has to be handled
            # separately 
            if(self.ndarr) :
                temp = self.convertToND(temp)
            try :
                setattr(target,self.name,temp)
            except AttributeError :
                print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
            except :
                raise
            self.addEtime(time.time())
            return
        # if data are numpy.ndarray then convert it
        if(self.type == type(ndarray)) :
            temp = ast.literal_eval(char)
            try :
                setattr(target,self.name,np.array(temp))
            except AttributeError :
                print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
            except :
                raise
            self.addEtime(time.time())
            return
        #print "SETTING ATTRIB"
        try :
            setattr(target,self.name,self.getData(char))
            self.addStime(time.time())
        except AttributeError :
            print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
        except :
             raise

    def close(self):
        return

    def getData(self,item):
        """
        Method used to convert string to designated type
        input: item - the string to convert
        """
        #print "GETTING DATA",item,self.type
        if(self.type == types.BooleanType) :
            return bool(item)
        if(self.type == types.IntType) :
            return int(item)
        if(self.type == types.LongType) :
            return long(item)
        if(self.type == types.FloatType) :
            return float(item)
        if(self.type == types.StringType) :
            return str(item)
        
    def convertToND(self,data):
        """
        Method to convert parts of lists/dictionaries to numpy.ndarrays
          Only parts of the input list/dictionary are converted, it is assumed that
          any list in the input data needs to be converted. It will only go 1 level
          deep looking for lists, so if deeper lists are necessary the code should
          be updated
        input: data - the list/dictionary that needs parts to be converted
        """
        temp = data
        if(self.type == types.ListType) :
            for i in range(0,len(temp)) :
                if(type(temp[i]) == types.ListType) :
                    temp[i] = np.array(temp[i])
            return temp
        for k,v in temp :
            if(type(v) == types.ListType) :
                temp[k] = np.array(v)
        return temp


class ATParser(sax.handler.ContentHandler) :
    """
        This is the customized parser for all BDP sub-classes
    """
    def __init__(self,xmlFile = "admit.xml") :
        self.start = time.time()
        self.ctime = time.time()
        self.etime = 0.0
        self.stime = 0.0
        self.ttime = 0.0
        self.sum = 0.0
        sax.handler.ContentHandler.__init__(self)
        self.pTime("Init",time.time())
        self.dtd = DTDParser(xmlFile)       # get the dtd info
        self.dtd.parse()
        self.pTime("DTD",time.time())
        self.AT = []
        self.curAT = None                     # an instance of the BDP sub-class
        self.type = None                    # the type of the BDP sub-class
        self.name = None                    # name of the current node
        self.ndarr = False                  # if the current node is an ndarray
        self.userData = None
        self.inAdmit = False
    
    def getAT(self):
        return self.AT
    
    def getUserData(self):
        return self.userData

    def addEtime(self,time):
        self.etime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
    
    def addTtime(self,time):
        self.ttime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
   
    def addStime(self,time):
        self.stime += (time - self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time
        
    def pTime(self,lab,time):
        print lab,(time-self.ctime)
        self.sum += (time - self.ctime)
        self.ctime = time

    def getAT(self):
        # return the generated BDP
        return copy.deepcopy(self.AT)

    def startElement(self, name, attrib) :
        """
        Method called whenever a new element is found
        dtd validation is done for each node and attribute
        input: name - the name of the element
               attrib - list of attributes found
        """
        # get the type of the data
        temp = attrib.get("type")
        if(ADMIT == name):
            self.dtd.check(name)
        elif(name == AT):
            print "AT ",temp,registry.ATTYPES
            print temp in registry.ATTYPES
            self.dtd.check(name, "type", temp.upper())
        else:
            self.dtd.check(name, "type", temp)
        self.name = name
        #print "\nNODE",name,temp
        if(temp == STRING) :
            self.type = types.StringType
        elif(temp == BOOL) :
            self.type = types.BooleanType
        elif(temp == FLOAT) :
            self.type = types.FloatType
        elif(temp == LONG) :
            self.type = types.LongType
        elif(temp == DICT) :
            self.type = types.DictType
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            try :
                if(attrib.get(int("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == LIST) :
            self.type = types.ListType
            #print "LIST",name
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            #print "LIST",name
            try :
                if(int(attrib.get("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == INT) :
            self.type = types.IntType
        elif(temp == TUPLE) :
            self.type = types.TupleType
            self.dtd.check(name, "ndarray", attrib.get("ndarray"))
            try :
                if(int(attrib.get("ndarray"))) :
                    self.ndarr = True
            except :
                pass
        elif(temp == NDARRAY) :
            self.type = type(ndarray)
        elif(name == AT and temp in registry.ATTYPES) :
            self.type = temp
            self.curAT = copy.deepcopy(registry.ATTYPES[temp])
            setattr(self.curAT,"id",str(attrib.get("id")))
            self.dtd.check(AT,"id",self.curAT.id)
            setattr(self.curAT,"parameters",ast.literal_eval(attrib.get("parameters")))
            self.dtd.check(AT,"parameters"," ")
            setattr(self.curAT,"version",str(attrib.get("version")))
            self.dtd.check(AT,"version",self.curAT.version)
            temp = attrib.get("stale")
            self.dtd.check(AT,"stale",temp)
            if(temp == "1"):
                setattr(self.curAT,"stale",True)
            else:
                setattr(self.curAT,"stale",False)
            #print "done start",temp,self.BDP
        elif(name == ADMIT) :
            self.inAdmit = True

    def endElement(self, name) :
        """
        Method called whenever the end of an xml element is reached
        input: name - the name of the element
        """
        # just reset the tracking stuff
        #print "end element",name
        self.type = None
        self.name = None
        self.ndarr = False
        if(name == AT) :
            self.inAdmit = False
            # one last validation run
            self.pTime("Start valid",time.time())
            self.AT.append(copy.deepcopy(self.curAT))
            self.curAT = None
        elif(name == ADMIT):
            if(not self.dtd.checkAll()) :
                print "Some required nodes missing from admit.xml file, attempting to continue anyway"


    
    def characters(self, ch) :
        """
        Method called whenever characters are detected in an xml node
        This method does some dtd validation
        input: ch - the characters detected
        """
        if(self.inAdmit):
            return
        target = None
        #print type(ch)
        char = str(ch).strip()
        #print "characters",char,"XXX"
        if(char.isspace() or not char) :
            #print "RETURNING"
            return
        # handle image, table, and task nodes specially
        target = self.curAT
        # a list or dictionary has to be decoded
        if(self.type == types.ListType or self.type == types.DictType or self.type == types.TupleType) :
            temp = ast.literal_eval(char)
            # if list/dict contains an numpy.ndarray it has to be handled
            # separately 
            if(self.ndarr) :
                temp = self.convertToND(temp)
            try :
                setattr(target,self.name,temp)
            except AttributeError :
                print "H1"
                print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
            except :
                raise
            self.addEtime(time.time())
            return
        # if data are numpy.ndarray then convert it
        if(self.type == type(ndarray)) :
            temp = ast.literal_eval(char)
            try :
                setattr(target,self.name,np.array(temp))
            except AttributeError :
                print "H2"
                print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
            except :
                raise
            self.addEtime(time.time())
            return
        #print "SETTING ATTRIB"
        try :
            setattr(target,self.name,self.getData(char))
            self.addStime(time.time())
        except AttributeError :
            print "H3"
            print "Data member %s is not a member of %s. This may be due to a version mismatch between the data and your software, attempting to continue." % (self.name,str(type(target)))
        except :
             raise

    def close(self):
        return

    def getData(self,item):
        """
        Method used to convert string to designated type
        input: item - the string to convert
        """
        #print "GETTING DATA",item,self.type
        if(self.type == types.BooleanType) :
            return bool(item)
        if(self.type == types.IntType) :
            return int(item)
        if(self.type == types.LongType) :
            return long(item)
        if(self.type == types.FloatType) :
            return float(item)
        if(self.type == types.StringType) :
            return str(item)
        
    def convertToND(self,data):
        """
        Method to convert parts of lists/dictionaries to numpy.ndarrays
          Only parts of the input list/dictionary are converted, it is assumed that
          any list in the input data needs to be converted. It will only go 1 level
          deep looking for lists, so if deeper lists are necessary the code should
          be updated
        input: data - the list/dictionary that needs parts to be converted
        """
        temp = data
        if(self.type == types.ListType) :
            for i in range(0,len(temp)) :
                if(type(temp[i]) == types.ListType) :
                    temp[i] = np.array(temp[i])
            return temp
        for k,v in temp :
            if(type(v) == types.ListType) :
                temp[k] = np.array(v)
        return temp


class DTDParser(object):
    """
    Class for parsing a dtd and holing the data for validation with xml
    """
    def __init__(self,xmlFile = None) :
        self.xmlFile = xmlFile      # the xml file containing the dtd
        self.entities = dict()          # list of dtd entities
        self.en = 0.0
        self.at = 0.0
        self.ctime = time.time()

    def prin(self) :
        for i in self.entities :
            self.entities[i].prin()

    def addEtime(self,time):
        self.en += (time - self.ctime)
        self.ctime = time
   
    def addAtime(self,time):
        self.at += (time - self.ctime)
        self.ctime = time
        
    def pTime(self,lab,time):
        print lab,(time-self.ctime)
        self.ctime = time

    def checkAll(self) :
        # check the dtd structure to see if all nodes were found
        #check = False
        for i in self.entities :
            print i
            if(not self.entities[i]["found"]) :
                print i,"not found"
                return False
            for a in self.entities[i]["attrib"] :
                print i,a
                if(not self.entities[i]["attrib"][a]["found"]) :
                    print i,a,"not found"
                    return False
        return True
    
    def parse(self,xmlFile = None) :
        # parse the dtd and generate the hierarchy
        if(self.xmlFile == None) :
            if(xmlFile == None) :
                raise Exception,"No xml file to parse"
            self.xmlFile = xmlFile
        f = open(self.xmlFile,'r')
        lines = f.readlines()
        self.pTime("Readlines",time.time())
        f.close()
        for line in lines :
            if("<!ELEMENT" in line) :
                self.ctime = time.time()
                en = line.split()
                #print en[1]
                self.entities[en[1]] = {"found":False,"attrib":{}}
                self.addEtime(time.time())
            elif("<!ATTLIST" in line) :
                self.ctime = time.time()
                at = line.split()
                values = at[3].replace(")","")
                values = values.replace("(","")
                values = values.split("|")
                #print at[1],at[2]
                self.entities[at[1]]["attrib"][at[2]] = {"found":False,"values":values}
                self.addAtime(time.time())
            elif("]>" in line) :
                print "Break"
                break
        #print self.entities
        print "En time",self.en
        print "At time",self.at
    
    def check(self,name,attrib = None, value =  None) :
        # check a node for validity
        print name,attrib,value
        try :
            if(name == "columns"):
                print "check ",name,attrib,value
            self.entities[name]["found"] = True
            if(attrib != None) :
                try :
                    if(not value in self.entities[name]["attrib"][attrib]["values"] and not "ANY" in self.entities[name]["attrib"][attrib]["values"]) :
                        print "Value %s for attribute %s is not a valid entry" % (value,name)
                        raise Exception,""
                    self.entities[name]["attrib"][attrib]["found"] = True
                except KeyError:
                    print "Attribute %s for %s not listed in DTD, malformed xml detected" % (attrib,name)
                    print "Inconsistency between dtd and xml detected, continuing"
                except :
                    print "Unknown error encountered while parsing attribute %s for %s" % (attrib,name)
                    raise
        except KeyError :
            print "Data member %s is not a member of the dtd, xml inconsistent with definition" % (name)
        except :
            raise
    
    def checkAttribute(self,name,attrib,value=None) :
        # check an attribute for validity
        if(name == "columns"):
                print "check ",name,attrib,value
        try :
            if(not value in self.entities[name]["attrib"][attrib]["values"] and not "ANY" in self.entities[name]["attrib"][attrib]["values"]) :
                print "Value %s for attribute %s is not a valid entry" % (value,self.name)
                raise Exception,""
            self.entities[name]["attrib"][attrib]["found"] = True
        except KeyError:
            print "Attribute %s not listed in DTD, malformed xml detected" % (attrib)
            print "Inconsistency between dtd and xml detected, continuing"
        except :
            print "Unknown error encountered while parsing attribute %s" % (attrib)
            raise

class BDP_ErrorHandler(sax.handler.ErrorHandler) :
    """
        Class for handling errors from the parser
    """
    def __init__(self) :
        pass
    
    def error(self,ex) :
        print "Recoverable error encountered: %s" % (ex.message)
        
    def fatalError(self,ex) :
        print "Fatal error encountered."
        raise ex
    
    def warning(self,ex) :
        print "Warning: %s" % (ex.message)

class AT_ErrorHandler(sax.handler.ErrorHandler) :
    """
        Class for handling errors from the parser
    """
    def __init__(self) :
        pass
    
    def error(self,ex) :
        print "Recoverable error encountered: %s" % (ex.message)
        
    def fatalError(self,ex) :
        print "Fatal error encountered."
        raise ex
    
    def warning(self,ex) :
        print "Warning: %s" % (ex.message)
